<?php
/*
Plugin Name: p01-contact
Description: Simply add contact forms in your pages
Version: 0.5
Author: Nicolas Liautaud
Author URI: http://nliautaud.fr

All informations here :
http://nliautaud.fr/wiki/travaux/getsimple#p01-contact

Use :
-----
  Place in a page the tag :
    (% contact %)
  To put a default contact form for the GS user address.
  
  You can define recipients by adding addresses separated by commas, like :
    (% contact : address@domain.ext, another@domain.ext... %)
 
  You can also construct your own form, by specifying the form fields and the
  fields options, separated by commas, like :
    (% contact : address@domain.ext, another@domain.ext,
                 a field and its options,
                 other field and its options,
                 another field and its options,
                 ... %)

Special fields :
----------------

These special fields types doing specific actions.

  - name    : a text field used as source name in email.
  - email   : a text field with email form verification, used as source address in email.
  - phone   : a text field with phone number verification.
  - website : a text field with web address verification.
  - subject : a text field used as subject in email.
  - message : a textarea with minimum text length verification, used as main content of the email.
  - captcha : a reloadable capcha image with text field entry verification.
  - askcopy : a checkbox allowing the user to receive a copy of the email.

General fields :
----------------

These fields are not doing anything special.

  - text     : a text field, for short information.
  - textarea : a textarea field, for longer information.
  - checkbox : a simple checkbox, for boolean information.

Fields options :
----------------

  - Exclamation point after field name define it as required : message!
  - Text between quotes define field title : message "Tell me something :"
  - Text after an equality symbol define field value : message = value
  - Text after an arrow symbol define field locked value : message => value

Place multiple options in this order : fieldType! "title" => value

Examples :
----------------

Minimal :
(% contact %)

Multi-recipients :
(% contact : recipient@domain.ext, other@domain.ext %)

Advanced form construction :
(% contact : recipient@domain.ext, other@domain.ext,
             subject => I found a bug,
             email!,
             text "Bug ID" = Change this only if you know what is it
             message! = Please tell the maximum (page address, browser used...),
             captcha!,
             askcopy %)

Settings :
----------*/

$settings = array(
     /* Enable or disable mail sending. */
    'enable'  => true,
     /* Force plugin language. Empty for GS installation language. */
    'lang'    => '',
     /* Fields used by default (when there is no tag parameters). */
    'default_fields' => array(
        'name' => array(
            'name' => 'name',
            'required' => 1,
            'title' => '',
            'locked' => 0,
            'value' => ''
            ),
        'email' => array(
            'name' => 'email',
            'required' => 1,
            'title' => '',
            'locked' => 0,
            'value' => ''
            ),
        'website' => array(
            'name' => 'website',
            'required' => 0,
            'title' => '',
            'locked' => 0,
            'value' => ''
            ),
        'subject' => array( 
            'name' => 'subject',
            'required' => 1,
            'title' => '',
            'locked' => 0,
            'value' => ''
            ),
        'message' => array(
            'name' => 'message',
            'required' => 1,
            'title' => '',
            'locked' => 0,
            'value' => ''
            ),
        'captcha' => array(
            'name' => 'captcha',
            'required' => 1,
            'title' => '',
            'locked' => 0,
            'value' => ''
            )
    ),
     /* Special fields types. */
     'fields_types' => array(
        'name'    => 'text',
        'email'   => 'text',
        'phone'   => 'text',
        'website' => 'text',
        'subject' => 'text',
        'address' => 'textarea',
        'message' => 'textarea',
        'captcha' => 'captcha',
        'askcopy' => 'checkbox'
    )
);


/************************************
 *             GSPLUGIN
 ***********************************/

$thisfile = basename(__FILE__, ".php");

register_plugin(
	$thisfile, 	            // ID of plugin, should be filename minus php
	'p01-contact', 	        // Title of plugin
	'0.5', 		            // Version of plugin
	'Nicolas Liautaud',	    // Author of plugin
	'http://nliautaud.fr', 	// Author URL
	'Simply add contact forms in your pages', // Plugin Description
	'plugins', 	            // Page type of plugin
	''  	                // Function that displays content
);

add_filter('content','content_p01contact');

function content_p01contact($contents)
{  
    global $settings;
    $tmp_content = $contents;
    $parameters = $targets = $forms_data = array();
    
    // find tags
    $pattern = '`(?<!<code>)\(%\s*contact\s*:?\s*(.*)\s*%\)`';  
    preg_match_all($pattern, $tmp_content, $tags, PREG_SET_ORDER);
    
    // find tags parameters
    $param_pattern = '`([^ ,!]+)(!)?\s*(&quot;([^&]*)&quot;)?\s*((=&gt;|=)?\s*([^,]*))?\s*`';
    foreach($tags as $id => $tag) {
        $forms_data[$id] = array('targets' => array(),
                                 'parameters' => array());
        $tag[1] = trim(preg_replace('`&nbsp;`',' ', $tag[1]));
        preg_match_all($param_pattern, $tag[1], $params, PREG_SET_ORDER);
        
        foreach($params as $param) {
            if(check_field_content('email', $param[0])) {
                array_push($forms_data[$id]['targets'], $param[0]);
            } else {
                $parameter['name'] = $param[1];
                $parameter['required'] = $param[2] == '!' ? 1 : 0;
                $parameter['title'] = $param[4];
                $parameter['locked'] = $param[6] == '=&gt;' ? 1 : 0;
                $parameter['value'] = $param[7];
                $forms_data[$id]['parameters'][$param[1]] = $parameter;
            }
        }
        // if there is no recipients, use admin address
        if(empty($forms_data[$id]['targets'])){
            $userdata = getXML(GSDATAOTHERPATH . 'user.xml');
            $forms_data[$id]['targets'][0] = $userdata->EMAIL;
        }
        // if there is no parameters, use default settings
        if(empty($forms_data[$id]['parameters'])){
            $forms_data[$id]['parameters'] = $settings['default_fields'];
        }
    }
    
    if(!empty($_POST['contact'])) {
        // store and check posted data
        $form_id = $_POST['contact_id'];
        $forms_data[$form_id]['fields'] = format_data($_POST['contact']);
        $forms_data[$form_id]['errors'] = check_form_content($forms_data[$form_id]);
        
        if(empty($forms_data[$form_id]['errors'])) {
            if($settings['enable'] == false) {
                // plugin is disable
                $forms_data[$form_id]['status'] = array(
                    'sent' => false,
                    'message' => 'plugin_disable'
                );
            } elseif(empty($forms_data[$form_id]['targets'])) {
                // there is no target
                $forms_data[$form_id]['status'] = array(
                    'sent' => false,
                    'message' => 'no_target'
                );
            } else {
                // send mail
                $forms_data[$form_id]['status'] = send_mail($forms_data[$form_id]);
            }
        } else {
            $forms_data[$form_id]['status']['sent'] = false;
        }
    }
    
    // modify content
    foreach($tags as $id => $tag) {
        $form = add_form($id, $forms_data[$id]);
        $tmp_content = preg_replace($pattern, $form, $tmp_content, 1);
    }
    
    return $tmp_content;
}



/************************************
 *            CHECKERS
 ***********************************/
 


function check_form_content($form_data)
{
    $error = array();
    foreach($form_data['fields'] as $name => $value) {
        if($value == null && $form_data['parameters'][$name]['required']) {
            // empty and required
            $error[$name] = 'required_field';
        }
        elseif($value != null && !check_field_content($name, $value)) {
            // not empty but not valid
            $error[$name] = 'error_' . $name;
        }
    }
    return $error;
}

function check_field_content($object, $value)
{
    switch($object) {
        case 'email':
            $pattern = '`^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$`i';
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'phone':
            $pattern = '`^\+?[-0-9(). ]{6,}$$`i';
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'website':
            $pattern = "`^((http|https|ftp):\/\/(www\.)?|www\.)[a-zA-Z0-9\_\-]+\.([a-zA-Z]{2,4}|[a-zA-Z]{2}\.[a-zA-Z]{2})(\/[a-zA-Z0-9\-\._\?\&=,'\+%\$#~]*)*$`i";
            if(preg_match($pattern, $value)) return 1;
            else return 0;
        case 'message':
            $size = strlen($value);
            if($size > 100) return 1;
            else return 0;
        case 'captcha':
            include_once 'plugins/p01-contact/captcha/securimage.php';
            $securimage = new Securimage();
            if ($securimage->check($value) == false) return 0;
            else return 1;   
        default:
            return 1;
    }
}

function field_required($name, $parameters) {
    global $settings;
    if( in_array($name, $settings['fields_required']) ||
        in_array($name.'!', $parameters))
        return true;
    else return false;
}



/************************************
 *         CREATE ELEMENTS
 ***********************************/


function add_form($id, $form_data)
{
    global $settings;

    $form  = '
    <form action="#p01-contact_' . $id . '" id="p01-contact_' . $id . '" class="p01-contact" method="post">';
    // display status message
    if(!empty($form_data['status']['message'])) {
        $class = $form_data['status']['sent'] ? 'updated' : 'error';
        $form .= '<div class="'.$class.'">' . lang($form_data['status']['message']) . '</div>';
    }
    // display settings fields or tag-defined fields ?
    if(!empty($form_data['parameters'])){
        $fields = $form_data['parameters'];
    } else {
        $fields = $settings['fields_shown'];
    }
    // add fields
    foreach($form_data['parameters'] as $field) {
        $form .= '<div>' . add_field($field, $form_data) . '</div>';
    }
    $form .= '<div><input name="contact_id" type="hidden" value="' . $id . '" />';
    $form .= '<input class="submit" id="contact-submit" name="contact-submit" type="submit" value="' . lang('send') . '" /></div>';
    $form .= '</form>';
    
    return $form;
}

function add_field($field, $form_data)
{
    global $settings;
    foreach($field as $key => $val) $$key = $val;
    if(specialfield_type($field)){
        $type = specialfield_type($field);
    } else {
        $type = $name;
    }

    $row  = '<label for="' . $name . '">';
    $row .= !empty($title) ? $title : lang($name);
    $row .= $required ? ' <strong>*</strong>' : '';
    $row .= '</label>';

    // fields error messages
    if(isset($form_data['errors'][$name])) {
        $row .= ' <span style="font-size:0.7em;color:red">';
        $row .= lang($form_data['errors'][$name]);
        $row .= '</span>';
    }

    // fill fields by precedent value
    if( isset($form_data['fields'][$name]) &&
        isset($form_data['status']['sent']) &&
        $form_data['status']['sent'] == false)
    {
        $value = $form_data['fields'][$name];
    }
    
    switch($type) {
        case 'textarea' :
            $row .= '<br/>';
            $row .= '<textarea id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" ';
            $row .=     'rows="10"';
            $row .=     $locked ? ' disabled="disabled"' : '';
            $row .= '>';
            $row .= $value . '</textarea>';
            break;
        case 'captcha' :
            $row .= '<br/>';
            $row .= '<img src="plugins/p01-contact/captcha/securimage_show.php" ';
            $row .=     'alt="CAPTCHA Image" /><br/>';
            $row .= '<a href="#"';
            $row .=     'onclick="document.getElementById(\'captcha\').src = ';
            $row .=     '\'plugins/p01-contact/captcha/securimage_show.php?\' ';
            $row .=     '+ Math.random(); return false">';
            $row .=     lang('reload');
            $row .= '</a> ';
            $row .= '<input id="' . $name . '" ';
            $row .=     'type="text" ';
            $row .=     'name="contact[' . $name . ']" ';
            $row .=     'size="10" maxlength="6"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .= ' />';
            break;
        case 'checkbox' :
            $row .= ' <input id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" ';
            $row .=     'type="checkbox"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .=     $value == 'on' ? 'checked' : '';
            $row .= ' />';
            break;
        case 'text' :
            $row .= '<br/>';
            $row .= '<input id="' . $name . '" ';
            $row .=     'name="contact[' . $name . ']" ';
            $row .=     'type="text" ';
            $row .=     'value="' . $value . '"';
            $row .=     $locked ? ' disabled="disabled" ' : ' ';
            $row .= ' />';
            break;
    }
    return $row;
}



function specialfield_type($field)
{
    global $settings;
    if(isset($settings['fields_types'][$field['name']])) {
        return $settings['fields_types'][$field['name']];
    } else {
        return 0;
    }
}


/************************************
 *              SEND
 ***********************************/


function send_mail($form_data)
{   
    $fields  = $form_data['fields'];
    $targets = implode(',', $form_data['targets']);
    $from    = $fields['email'];
    $subject = '=?utf-8?B?' . base64_encode($fields['subject']) . '?=';
    
    $content  = '<h1>' . lang('sentfrom') . ' ' . get_page_url(1) . '</h1>';
    $content .= '<ul><li>' . lang('date').' : <b>' . date('r') . '</b></li>';
    
    $skip = array('message', 'captcha');
    foreach($fields as $name => $value) {
        if(!in_array($name, $skip) && !empty($value)) {
            $content .= '<li>' . lang($name).' : <b>' . $value . '</b></li>';
        }
    }
    $content .= '</ul><p>' . nl2br($fields['message']) . '</p>';
    
    $headers  = "From: $from\r\n";
    $headers .= "Reply-To: $from\r\n";
    $headers .= "Return-Path: $from\r\n";
    $headers .= "MIME-Version: 1.0\r\n";
    $headers .= "Content-type: text/html; charset=UTF-8\r\n";
    $headers .= "X-Mailer: PHP/" . phpversion() . "\r\n" ;

    $sent = mail($targets, $subject, $content, $headers);
    if( isset($fields['askcopy']) &&
        $fields['askcopy'])
        $copy = mail($from, $subject, $content, $headers);
    
    if($sent) $status = array('sent' => true, 'message' => 'sent');
    else $status = array('sent' => false, 'message' => 'error_send');
    return $status;
}


/************************************
 *             TOOLS
 ***********************************/


function format_data($array)
{
    foreach($array as $key => $val){
        $tmp = stripslashes($val);
        $tmp = htmlentities($tmp, ENT_QUOTES, 'UTF-8');
        $array[$key] = $tmp;
    }
    return $array;
}


/************************************
 *             LANG
 ***********************************/


function lang($key)
{
    global $settings, $LANG;
    
    $langs = array('en', 'fr');
    $words = array(
        // UI
        'name'     => array('en' => 'Name',            'fr' => 'Nom'),
        'email'    => array('en' => 'Email',           'fr' => 'Adresse email'),
        'address'  => array('en' => 'Postal address',  'fr' => 'Adresse postale'),
        'phone'    => array('en' => 'Phone number',    'fr' => 'Numéro de téléphone'),
        'website'  => array('en' => 'Website',         'fr' => 'Site web'),
        'subject'  => array('en' => 'Subject',         'fr' => 'Objet'),
        'message'  => array('en' => 'Message',         'fr' => 'Message'),
        'captcha'  => array('en' => 'Captcha',         'fr' => 'Captcha'),
        'askcopy'  => array('en' => 'Send me a copy of this email',         'fr' => 'Envoyez-moi une copie de cet email'),
        'send'     => array('en' => 'Send',            'fr' => 'Envoyer'),
        'sentfrom' => array('en' => 'Sent from',       'fr' => 'Envoyé depuis'),
        'date'     => array('en' => 'Date',            'fr' => 'Date'),
        'sent'     => array('en' => 'Email sent.',     'fr' => 'Email envoyé.'),
        'reload'   => array('en' => 'Reload',          'fr' => 'Recharger'),
        
        // general messages
        'plugin_disable'=> array('en' => 'The p01-contact plugin is disable. No message was sent.',
                                 'fr' => 'Le plugin p01-contact est désactivé. Aucun message n\'a été envoyé.'),
        'no_target'     => array('en' => 'This contact form has not addressed. No message was sent.',
                                 'fr' => 'Ce formulaire de contact n\'a pas de destinataire. Aucun message n\'a été envoyé.'),
        'error_send'    => array('en' => 'Error : message not sent.',
                                 'fr' => 'Erreur : le message n\'a pas été envoyé.'),
        // fields errors messages
        'required_field'=> array('en' => 'This field is required',
                                 'fr' => 'Ce champ est obligatoire'),
        'error_email'   => array('en' => 'Please use a valid email address',
                                 'fr' => 'Veuillez entrer une adresse email valide'),
        'error_phone'   => array('en' => 'Please use a valid phone number',
                                 'fr' => 'Veuillez entrer un numéro de téléphone valide'),
        'error_website' => array('en' => 'Please use a valid web address',
                                 'fr' => 'Veuillez entrer une adresse valide'),
        'error_message' => array('en' => 'Please write a longer message',
                                 'fr' => 'Veuillez écrire un message plus long'),
        'error_captcha' => array('en' => 'Please copy the following text',
                                 'fr' => 'Veuillez recopier le texte ci-dessous')
    );
    
    // if language not defined in settings, use GS install language.
    $lang = empty($settings['lang']) ? substr($LANG, 0, 2) : $settings['lang'];
    
    // if language not traducted, use en
    $lang = in_array($lang, $langs) ? $lang : 'en';
    
    if (isset($words[$key][$lang])) {
        return $words[$key][$lang];
    } else {
        return ucfirst($key);
    }
}
